import math
from ummalqura_arrray import UmalqurraArray
'''
This class is responsoble to convert from Hijri to Gregorian or from Gregorian to Hijri
The algrothem was converted  from java script to python by Khalid Al-hussayen in 1436-3-14 2015-1-5
The orjinal source developed by Suhail Alkowaileet the source url https://github.com/xsoh/Hijri.js/blob/master/Hijri.js 
'''

class Umalqurra:
    def gegorean_to_hijri(self, year, month, day):
        # This code the modified version of R.H. van Gent Code, it can be found at http://www.staff.science.uu.nl/~gent0113/islam/ummalqura.htm
        # read calendar data
        day = int(day)
        m = int(month)  # Here we enter the Index of the month (which starts with Zero)
        y = int(year)
        # append January and February to the previous year (i.e. regard March as
        # the first month of the year in order to simplify leapday corrections)
        if m < 3:
            y -= 1
            m += 12
            # determine offset between Julian and Gregorian calendar
        a = math.floor(y / 100.)
        jgc = a - math.floor(a / 4.) - 2
        # compute Chronological Julian Day Number (CJDN)
        cjdn = math.floor(365.25 * (y + 4716)) + math.floor(30.6001 * (m + 1)) + day - jgc - 1524
        a = math.floor((cjdn - 1867216.25) / 36524.25)
        # compute Modified Chronological Julian Day Number (MCJDN)
        mcjdn = cjdn - 2400000
        #the MCJDN's of the start of the lunations in the Umm al-Qura calendar are stored in 'islamcalendar_dat.js'
        index = UmalqurraArray.get_index(mcjdn)
        # compute and output the Umm al-Qura calendar date
        iln = index + 16260
        ii = math.floor((iln - 1) / 12)
        iy = ii + 1
        im = iln - 12 * ii
        id = mcjdn - UmalqurraArray.ummalqura_dat[index - 1] + 1
        return iy, im, id

    def hijri_to_gregorian(self, year, month, day):
        year = int(year)
        month = int(month)
        day = int(day)
        iy = year
        im = month
        id = day
        ii = iy - 1
        iln = (ii * 12) + 1 + (im - 1)
        i = iln - 16260
        mcjdn = id + UmalqurraArray.ummalqura_dat[i - 1] - 1
        cjdn = mcjdn + 2400000
        return self.julianToGregorian(cjdn);


    def julianToGregorian(self,julianDate):
        # source from: http://keith-wood.name/calendars.html
        z = math.floor(julianDate + 0.5)
        a = math.floor((z - 1867216.25) / 36524.25)
        a = z + 1 + a - math.floor(a / 4)
        b = a + 1524;
        c = math.floor((b - 122.1) / 365.25)
        d = math.floor(365.25 * c)
        e = math.floor((b - d) / 30.6001)
        day = b - d - math.floor(e * 30.6001)
        if e > 13.5:
            month = e - 13
        else:
            month = e - 1
        if month > 2.5:
            year = c - 4716
        else:
            year = c - 4715
        if year <= 0:
            year -= 1
        return year, month , day



# -*- coding: utf-8 -*-
'''
This is an API for Hijri Umalqurra Calendar,
it was developed  by khalid Al-hussayen 1436-3-15 2015-1-6.
The algrothem of converting hijri to  Gregorian is in hijri.py
This Api will give the ability to convert Gregorian To Hijri or Hijri to Gregorian with the day and month names in Hijri and Gregorian
You can query for the current day in both Hijri and Gregorian
'''
__author__ = 'Khalid'
from hijri import Umalqurra
from datetime import date
class HijriDate:
    #day in hijri
    day = -1
    #month in hijri
    month = -1
    #year in hijri
    year = -1
    day_gr = -1
    month_gr = -1
    year_gr = -1
    #day bane in arabic
    day_name = ''
    #month name in hijri
    month_name = ''
    month_dict = {1:'محرم',2:'صفر',3:'ربيع الأول',4:'ربيع الثاني',5:'جمادي الأولى',6:'جمادي الآخرة',7:'رجب'
    ,8:'شعبان',9:'رمضان',10:'شوال',11:'ذو القعدة',12:'ذو الحجة'}
    day_dict = {'Saturday':'السبت','Sunday':'الاحد','Monday':'الاثنين','Tuesday':'الثلاثاء',
                'Wednesday':'الاربعاء','Thursday':'الخميس','Friday':'الجمعة'}
    month_name_gr = ''
    day_name_en = ''
    def __init__(self,year=None,month=None,day=None,gr=False):
        if year != None and month != None and day != None:
            if gr == False:
                self.set_date(year,month,day)
            else:
                self.set_date_from_gr(year,month,day)
    #Set dates if the date send by user is Gregorian
    def set_date_from_gr(self,year,month,day):
        um = Umalqurra()
        self.day_gr, self.month_gr, self.year_gr = day, month, year
        self.year, self.month, self.day = um.gegorean_to_hijri(year,month,day)
        self.month_name = self.month_dict[self.month]
        date_gr = date(year,month,day)
        self.day_name_en = date_gr.strftime("%A")
        self.day_name = self.day_dict[self.day_name_en]
        self.month_name_gr = date_gr.strftime("%B")
    #Set dates if date send by user is Hijri
    def set_date(self,year,month,day):
        um = Umalqurra()
        self.day, self.month, self.year = day, month, year
        self.month_name = self.month_dict[month]
        self.year_gr, self.month_gr, self.day_gr = um.hijri_to_gregorian(year,month,day)
        date_gr = date(int(self.year_gr),int(self.month_gr),int(self.day_gr))
        self.day_name_en = date_gr.strftime("%A")
        self.day_name = self.day_dict[self.day_name_en]
        self.month_name_gr = date_gr.strftime("%B")

    @classmethod
    def today(cls):
        today = date.today()
        hijri_date = HijriDate(today.year,today.month,today.day,True)
        return hijri_date